@@ -33,7 +33,10 @@ import           NameSet (NameSet)
 import           Coercion (Coercion)
 #endif
 
+#if __GLASGOW_HASKELL__ < 805
 import           FastString (unpackFS)
+#endif
+
 import           Digraph (flattenSCCs)
 
 import           System.Posix.Internals (c_getpid)
@@ -76,6 +79,21 @@ data Module a = Module {
 instance NFData a => NFData (Module a) where
   rnf (Module name setup content) = name `deepseq` setup `deepseq` content `deepseq` ()
 
+#if __GLASGOW_HASKELL__ < 803
+type GhcPs = RdrName
+
+needsTemplateHaskellOrQQ :: ModuleGraph -> Bool
+needsTemplateHaskellOrQQ = needsTemplateHaskell
+
+mapMG :: (ModSummary -> ModSummary) -> ModuleGraph -> ModuleGraph
+mapMG = map
+#endif
+
+#if __GLASGOW_HASKELL__ < 805
+addQuoteInclude :: [String] -> [String] -> [String]
+addQuoteInclude includes new = new ++ includes
+#endif
+
 -- | Parse a list of modules.
 parse :: [String] -> IO [TypecheckedModule]
 parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
@@ -86,7 +104,7 @@ parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
   mapM (`guessTarget` Nothing) modules >>= setTargets
   mods <- depanal [] False
 
-  mods' <- if needsTemplateHaskell mods then enableCompilation mods else return mods
+  mods' <- if needsTemplateHaskellOrQQ mods then enableCompilation mods else return mods
 
   let sortedMods = flattenSCCs (topSortModuleGraph False mods' Nothing)
   reverse <$> mapM (parseModule >=> typecheckModule >=> loadModule) sortedMods
@@ -103,7 +121,7 @@ parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
       modifySessionDynFlags enableComp
       -- We need to update the DynFlags of the ModSummaries as well.
       let upd m = m { ms_hspp_opts = enableComp (ms_hspp_opts m) }
-      let modGraph' = map upd modGraph
+      let modGraph' = mapMG upd modGraph
       return modGraph'
 
     -- copied from Haddock/GhcUtils.hs
@@ -141,7 +159,7 @@ parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
         objectDir  = Just f
       , hiDir      = Just f
       , stubDir    = Just f
-      , includePaths = f : includePaths d
+      , includePaths = addQuoteInclude (includePaths d) [f]
       }
 
 -- | Extract all docstrings from given list of files/modules.
@@ -174,7 +192,7 @@ extractFromModule m = Module name (listToMaybe $ map snd setup) (map snd docs)
 
 -- | Extract all docstrings from given module.
 docStringsFromModule :: ParsedModule -> [(Maybe String, Located String)]
-docStringsFromModule mod = map (fmap (toLocated . fmap unpackDocString)) docs
+docStringsFromModule mod = map (fmap (toLocated . fmap unpackHDS)) docs
   where
     source   = (unLoc . pm_parsed_source) mod
 
@@ -187,8 +205,10 @@ docStringsFromModule mod = map (fmap (toLocated . fmap unpackDocString)) docs
     header  = [(Nothing, x) | Just x <- [hsmodHaddockModHeader source]]
 #if __GLASGOW_HASKELL__ < 710
     exports = [(Nothing, L loc doc) | L loc (IEDoc doc) <- concat (hsmodExports source)]
-#else
+#elif __GLASGOW_HASKELL__ < 805
     exports = [(Nothing, L loc doc) | L loc (IEDoc doc) <- maybe [] unLoc (hsmodExports source)]
+#else
+    exports = [(Nothing, L loc doc) | L loc (IEDoc _ doc) <- maybe [] unLoc (hsmodExports source)]
 #endif
     decls   = extractDocStrings (hsmodDecls source)
 
@@ -236,13 +256,17 @@ extractDocStrings = everythingBut (++) (([], False) `mkQ` fromLHsDecl
 #endif
   )
   where
-    fromLHsDecl :: Selector (LHsDecl RdrName)
+    fromLHsDecl :: Selector (LHsDecl GhcPs)
     fromLHsDecl (L loc decl) = case decl of
 
       -- Top-level documentation has to be treated separately, because it has
       -- no location information attached.  The location information is
       -- attached to HsDecl instead.
+#if __GLASGOW_HASKELL__ < 805
       DocD x -> select (fromDocDecl loc x)
+#else
+      DocD _ x -> select (fromDocDecl loc x)
+#endif
 
       _ -> (extractDocStrings decl, True)
 
@@ -257,6 +281,8 @@ extractDocStrings = everythingBut (++) (([], False) `mkQ` fromLHsDecl
       DocCommentNamed name doc -> (Just name, L loc doc)
       _                        -> (Nothing, L loc $ docDeclDoc x)
 
+#if __GLASGOW_HASKELL__ < 805
 -- | Convert a docstring to a plain string.
-unpackDocString :: HsDocString -> String
-unpackDocString (HsDocString s) = unpackFS s
+unpackHDS :: HsDocString -> String
+unpackHDS (HsDocString s) = unpackFS s
+#endif
