@@ -6,8 +6,10 @@ main :: IO ()
 main = doctest [
     "-packageghc"
   , "-isrc"
+  , "-ighci-wrapper/src"
   , "-idist/build/autogen/"
   , "-optP-include"
   , "-optPdist/build/autogen/cabal_macros.h"
   , "src/Run.hs"
+  , "src/PackageDBs.hs"
   ]
